# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['botocove']

package_data = \
{'': ['*']}

install_requires = \
['boto3', 'boto3-stubs[sts,organizations]', 'tqdm']

setup_kwargs = {
    'name': 'botocove',
    'version': '1.5.2',
    'description': 'A decorator to allow running a function against all AWS accounts in an organization',
    'long_description': '# Botocove\n\nRun a function against a selection of AWS accounts, or all AWS accounts in an\norganization, concurrently with thread safety. Run in one or multiple regions.\n\nBy default, opinionated to work with the standard\nAWS Organization master/member configuration from an organization master\naccount.\n\n- Fast\n- Easy\n- Dolphin Themed 🐬\n\nA simple decorator for functions to remove time and complexity burden. Uses\n`ThreadPoolExecutor` to run boto3 sessions against AWS accounts concurrently.\n\nDecorating a function in `@cove` provides a boto3 session to the function and runs it\nin every account required, gathering all results into a dictionary.\n\n**Warning**: this tool gives you the potential to make dangerous changes\nat scale. **Test carefully and make idempotent changes**! Please read available\narguments to understand safe experimentation with this package.\n\n## Pre-requisites and Info\n\nAn IAM user with `sts:assumerole` privilege, and accounts that have a trust\nrelationship to the IAM user\'s account.\n\nBy default, the IAM user is expected to be in an AWS Organization Master account. You can alter nearly all behaviour of Cove with appropriate [arguments](#arguments)\n\nCove will not execute a function call in the account it\'s called from.\n\nDefault requirements are:\n\nIn the organization master account:\n* IAM permissions `sts:assumerole`, `sts:get-caller-identity` and\n`organizations:list-accounts`\n\nIn the organization accounts:\n* An `AccountOrganizationAccessRole` role\n\nSee the [arguments](#arguments) section for how to change these defaults to work with any\naccount configuration, including running without an AWS Organization.\n\n## Quickstart\nA function written to interact with one\n[boto3 session](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html)\ncan now be called with a `session` from every account and region required by assuming a\n role in for each account - except the host account you\'re running from.\n\nFor example:\n\nA standard approach: this function takes a boto3 `session` and gets all IAM\nusers from a single AWS account. You would then manually run it in each account.\n\n```\nimport boto3\n\n\ndef get_iam_users(session):\n    iam = session.client("iam", region_name="eu-west-1")\n    all_users = iam.get_paginator("list_users").paginate().build_full_result()\n    return all_users\n\ndef main():\n    session = boto3.session.Session(profile_name="my_dev_account")\n    users = get_iam_users(session)\n    print(users) # A single account\'s IAM users\n```\n\nNow with `@cove`: a session for every account in the organization is injected\nby the decorator. A safe example to run as a test!\n\n```\nimport boto3\nfrom botocove import cove\n\n@cove()\ndef get_iam_users(session):\n    iam = session.client("iam", region_name="eu-west-1")\n    all_users = iam.get_paginator("list_users").paginate().build_full_result()\n    return all_users\n\ndef main():\n    # No session passed as the decorator injects it\n    all_results = get_iam_users()\n    # Now returns a Dict with keys Results, Exceptions and FailedAssumeRole\n\n    # A list of dictionaries for each account, with account details included.\n    # Each account\'s get_iam_users return is in a "Result" key.\n    print(all_results["Results"])\n\n    # A list of dictionaries for each account that raised an exception\n    print(all_results["Exceptions"])\n\n    # A list of dictionaries for each account that could not be assumed into\n    print(all_results["FailedAssumeRole"])\n```\n\nHere\'s an example of a more customised Cove decorator:\n```\n@cove(\n    target_ids=["123456789101", "234567891011"],\n    rolename="AWSControlTowerExecution",\n    raise_exception=True,\n    regions=["eu-west-1", "eu-west-2", "us-east-1"],\n)\ndef do_things(session):\n    # Cove will return six results of True, 2 accounts * 3 regions\n    return True\n```\n## Arguments\n\n### Cove\n`@cove()`:\n\nUses the standard boto3 credential chain to start with, assuming roles in every account\nrequired. Defaults to assuming the `OrganizationAccountAccessRole` in every account\nin an AWS organization.\n\nEquivalent to:\n`@cove(target_ids=None, ignore_ids=None, rolename=None, role_session_name=rolename,\n    policy=None, policy_arns=None, assuming_session=None, raise_exception=False,\n    org_master=True, thread_workers=20, regions=None)`\n\n`target_ids`: List[str]\n\nA list of AWS accounts as strings to attempt to assume role in to. When unset,\ndefault attempts to use every available account ID in an AWS organization.\n\n`ignore_ids`: List[str]\n\nA list of AWS account ID\'s that will not attempt assumption in to. Allows IDs to\nbe ignored.\n\n`rolename`: str\n\nAn IAM role name that will be attempted to assume in all target accounts.\nDefaults to the AWS Organization default, `OrganizationAccountAccessRole`.\n\n`role_session_name`: str\n\nAn IAM role session name that will be passed to each Cove session\'s `sts.assume_role()` call.\nDefaults to the name of the role being used if unset.\n\n`policy`: str\n\nA policy document that will be used as a session policy in each Cove session\'s `sts.assume_role()`\ncall. Unless the value is None, it is passed through via the Policy parameter.\n\n`policy_arns`: List[str]\n\nA list of managed policy ARNs that will be used as a session policy in each Cove session\'s\n `sts.assume_role()` call. Unless the value is None, it is passed through via the PolicyArns parameter.\n\n`assuming_session`: Session\n\nA Boto3 `boto3.session.Session()` object that will be used to call `sts:assumerole`. If not\nprovided, cove will instantiate one which will use the standard boto3 credential chain.\n\n`raise_exception`: bool\n\nDefaults to False. Default behaviour is that exceptions are not raised from\ndecorated function. This is due to `cove` running asynchronously and preferring\nto resolve all tasks and report their results instead of exiting early.\n\n`raise_exception=True` will allow a full stack trace to escape on the first\nexception seen; but will not gracefully or consistently interrupt running tasks.\nIt is vital to run interruptible, idempotent code with this argument as `True`.\n\n`org_master`: bool\n\nDefaults to True. When True, will leverage the Boto3 Organizations API to list\nall accounts in the organization, and enrich each `CoveSession` with information\navailable (`Id`, `Arn`, `Name`, `Status`, `Email`). Disabling this and providing your\nown full list of accounts may be a desirable optimisation if speed is an issue.\n\n`org_master=False` means only `Id` will be\navailable to `CoveSession`.\n\n`thread_workers`: int\n\nDefaults to 20. Cove utilises a ThreadPoolWorker under the hood, which can be tuned\nwith this argument. Number of thread workers directly corrolates to memory usage: see\n[here](#is-botocove-thread-safe)\n\n`regions`: List[str]\n\nIf not provided, Cove will respect your profile\'s default region via the boto credential chain.\nIf provided, Cove will run the decorated function in every region named.\n\nYou can get all regions with:\n`regions = [r[\'RegionName\'] for r in boto3.client(\'ec2\').describe_regions()[\'Regions\']]`✨\n\n### CoveSession\n\nCove supplies an enriched Boto3 session to each function called. Account details\nare available with the `session_information` attribute.\n\nOtherwise, it functions exactly as calling `boto3` would.\n\n```\n@cove()\ndef do_nothing(session: CoveSession):\n    print(session.session_information) # Outputs a dict of known information\n    # This function runs no boto3-specific API, but does test that a role\n    # can be assumed\n```\n\n## Return values\n\nWrapped functions return a dictionary. Each value contains List[Dict[str, Any]]:\n```\n{\n    "Results": results:\n    "Exceptions": exceptions,\n    "FailedAssumeRole": invalid_sessions,\n}\n```\nAn example of cove_output["Results"]:\n```\n[ # A list of dictionaries per account called\n    {\n    \'Id\': \'123456789010\',\n    \'Email\': \'email@address.com\',\n    \'Name\': \'account-name\',\n    \'Status\': \'ACTIVE\',\n    \'AssumeRoleSuccess\': True,\n    \'Result\': wrapped_function_return_value # Result of wrapped func\n    }\n]\n```\n\n### Is botocove thread safe?\n\nbotocove is thread safe, but number of threaded executions will be bound by memory,\nnetwork IO and AWS api rate limiting. Defaulting to 20 thread workers is a reasonable\nstarting point, but can be further optimised for runtime with experimentation.\n\nbotocove has no constraint or understanding of the function it\'s wrapping: it is\nrecommended to avoid shared state for botocove wrapped functions, and to write simple\nfunctions that are written to be idempotent and independent.\n\n[Boto3 Session objects are not natively thread safe and should not be shared across threads](https://boto3.amazonaws.com/v1/documentation/api/1.14.31/guide/session.html#multithreading-or-multiprocessing-with-sessions).\nHowever, botocove is instantiating a new Session object per thread/account and running\ndecorated functions inside their own closure. A shared client is used from the host account\nthat botocove is run from (eg, an organization master account) -\n[clients are threadsafe](https://boto3.amazonaws.com/v1/documentation/api/latest/guide/clients.html#multithreading-or-multiprocessing-with-clients) and allow this.\n\nboto3 sessions have a significant memory footprint:\nVersion 1.5.0 of botocove was re-written to ensure that boto3 sessions are released\nafter completion which resolved memory starvation issues. This was discussed here:\nhttps://github.com/connelldave/botocove/issues/20 and a relevant boto3 issue is here:\nhttps://github.com/boto/boto3/issues/1670\n\n### botocove?\n\nIt turns out that the Amazon\'s Boto dolphins are solitary or small-group animals,\nunlike the large pods of dolphins in the oceans. This killed my "large group of\nboto" idea, so the next best idea was where might they all shelter together... a\ncove!\n',
    'author': 'Dave Connell',
    'author_email': 'daveconn41@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/connelldave/botocove',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
